% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{is.gtable}
\alias{is.gtable}
\title{Is this a gtable?}
\usage{
is.gtable(x)
}
\arguments{
\item{x}{object to test}
}
\description{
Is this a gtable?
}
